"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
function promisify(generator) {
    // Implementation based on https://github.com/tj/co/blob/master/index.js
    return function () {
        var args = arguments;
        var ctx = this;
        var gen = generator.apply(ctx, args);
        var rejector;
        var currentPromise = undefined;
        var res = new Promise(function (resolve, reject) {
            rejector = reject;
            function onFulfilled(res) {
                currentPromise = undefined;
                var result;
                try {
                    result = gen.next(res);
                }
                catch (e) {
                    reject(e);
                    return;
                }
                // noinspection JSIgnoredPromiseFromCall
                next(result);
            }
            function onRejected(err) {
                currentPromise = undefined;
                var result;
                try {
                    result = gen.throw(err);
                }
                catch (e) {
                    reject(e);
                    return;
                }
                // noinspection JSIgnoredPromiseFromCall
                next(result);
            }
            function next(ret) {
                if (ret && typeof ret.then === "function") {
                    // an async iterator
                    ret.then(next, reject);
                    return;
                }
                if (ret.done) {
                    resolve(ret.value);
                    return;
                }
                currentPromise = Promise.resolve(ret.value);
                return currentPromise.then(onFulfilled, onRejected);
            }
            onFulfilled(undefined); // kick off the process
        });
        res.cancel = function () {
            try {
                if (currentPromise) {
                    cancelPromise(currentPromise);
                }
                var res_1 = gen.return();
                // eat anything that promise would do, it's cancelled!
                var yieldedPromise = Promise.resolve(res_1.value);
                yieldedPromise.then(function () {
                    /* nothing */
                }, function () {
                    /* nothing */
                });
                cancelPromise(yieldedPromise); // maybe it can be cancelled :)
                rejector(new Error("PROMISE_CANCELLED"));
            }
            catch (e) {
                rejector(e); // there could be a throwing finally block
            }
        };
        return res;
    };
}
exports.promisify = promisify;
function cancelPromise(promise) {
    if (typeof promise.cancel === "function") {
        promise.cancel();
    }
}
//# sourceMappingURL=Utils.js.map